import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:transparent_image/transparent_image.dart';

enum ContentCardVariant { mini, nano }

/// Создаёт карточку, предназначенную для работы с тематическими сборниками.
class ContentCard extends StatelessWidget {
  final String title;
  final String description;
  final ImageProvider image;
  final String imageUrl;
  final Color textColor;
  final double height;
  final double width;
  final ContentCardVariant variant;

  ContentCard({
    Key key,
    this.title,
    this.description,
    this.image,
    this.imageUrl,
    this.textColor,
    this.height,
    this.width,
    this.variant,
  })  : assert(image == null || imageUrl == null),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return _buildBody(
      context,
      child: InkWell(
        onTap: () {},
        child: Stack(
          children: <Widget>[
            if (imageUrl != null) _buildBackgroundImage(context),
            if (variant != ContentCardVariant.nano) _buildContent(context),
          ],
        ),
      ),
    );
  }

  /// Создаёт содержимое карточки.
  Widget _buildContent(BuildContext context) {
    return Container(
      padding: EdgeInsets.only(bottom: 20),
      alignment: Alignment.bottomLeft,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          SizedBox(width: 20),
          Expanded(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                if (description != null && variant != ContentCardVariant.mini)
                  _buildDescription(context),
                _buildCaption(context),
              ],
            ),
          ),
          SizedBox(width: 20),
        ],
      ),
    );
  }

  /// Создаёт заголовок карточки.
  Widget _buildCaption(BuildContext context) {
    return Text(
      title,
      style: Theme.of(context).textTheme.headline6.copyWith(
          fontSize: variant != ContentCardVariant.mini ? 17 : 15,
          fontWeight: FontWeight.w500,
          color: textColor ?? Colors.white),
    );
  }

  /// Создаёт описание карточки.
  Widget _buildDescription(BuildContext context) {
    return Padding(
      padding: EdgeInsets.only(bottom: 1),
      child: Opacity(
        opacity: 0.7,
        child: Text(
          description,
          style: Theme.of(context).textTheme.headline6.copyWith(
              fontSize: 12,
              fontWeight: FontWeight.w500,
              color: textColor ?? Colors.white),
        ),
      ),
    );
  }

  /// Создаёт фон карточки.
  Widget _buildBackgroundImage(BuildContext context) {
    return Stack(
      fit: StackFit.expand,
      children: <Widget>[
        Transform.scale(
          scale: 1.001,
          child: image != null && imageUrl == null
              ? Image(
                  image: image,
                  fit: BoxFit.cover,
                )
              : FadeInImage.memoryNetwork(
                  fadeInDuration: Duration(milliseconds: 150),
                  placeholder: kTransparentImage,
                  image: imageUrl,
                  fit: BoxFit.cover,
                ),
        ),
        Container(
          color: Colors.black.withOpacity(0.6),
        )
      ],
    );
  }

  /// Основа карточки.
  Widget _buildBody(BuildContext context, {Widget child}) {
    return SizedBox(
      width: width ?? double.infinity,
      height: height ?? double.infinity,
      child: Opacity(
        opacity: variant != ContentCardVariant.nano ? 1 : 0.4,
        child: Material(
          color: Theme.of(context).cardColor,
          borderRadius: BorderRadius.circular(15),
          clipBehavior: Clip.antiAlias,
          child: child,
        ),
      ),
    );
  }
}
